//--------------------------------------------------------------------------------------
// JCSDK.h
// Client SDK Interface Definition
//--------------------------------------------------------------------------------------

#pragma once

#ifdef __cplusplus
#  define JCSDK_API extern "C" __declspec(dllexport)
#else
#  define JCSDK_API __declspec(dllexport)
#endif

#define JCSDK_CALLTYPE __stdcall

#define JCSDK_INVALID_LINKVALUE -1
typedef int JCLink_t;

#define JCMAX_LINKNUM 512

//Event Type
enum JCEventType
{
	//Connecting
	JCET_Connected, //Unused
	JCET_ConnectOK,	//Successful connection
	JCET_UserAccessError, //User authentication failed
	JCET_NoChannel,		//Master channel not open
	JCET_ConTypeError,	//Connecting type wrong
	JCET_ConCountLimit,	//Over the maximum of master channel connection
	JCET_ConTimeout,	//Connection timeout
	JCET_DisconOK, //Disconnect
	JCET_ConAbout, //Abnormal disconnect
	JCET_ServiceStop,	//Master disconnect

	//Remote video
	JCET_GetFileListOK,		//Successfully get remote video
	JCET_GetFileListError,	//Failed to get remote video

	JCET_RemotePlayBegin,	//Remote playback start
	JCET_RemotePlayEnd,		//Remote playback end
	JCET_RemoteError,		//Remote playback failed
	JCET_RemotePlayTimeout,	//Remote playback timeout

	JCET_DownloadData,		//Video data download
	JCET_DownloadEnd,		//Video download finished
	JCET_DownloadStop,		//Video download stop
	JCET_DownloadError,		//Video download failed
	JCET_DownloadTimeout,	//Remote download timeout

	//Other
	JCET_StreamReset,	//Stream reset (It is need to close and reopen the decoding,video, etc for changing the stream configurqation. )
	JCET_Alarm,	//Alarm
};

//Connection
enum JCLinkStatus
{
	JCLS_UnConnect,	//Unconnected
	JCLS_Connecting,//connecting
	JCLS_WaitData,	//Successful connection,waiting for data
	JCLS_Connected,	//Connected
};

//Frame Type
enum JCFrameType
{
	JCFT_I,			//Video frame I
	JCFT_P,			//Video frame P
	JCFT_B,			//Video frame B
	JCFT_Audio,		//Audio frame
};

//Record type
enum JCRecFileType
{
	JCRT_Unknown = 0,// Unknown
	JCRT_SV4,		//sv4
	JCRT_SV5,		//sv5
	JCRT_SV6,		//sv6
	JCRT_MP4,		//MP4
};

//Device Type
enum JCDeviceType
{
	JCDT_All = 0,		//Unknown Device
	JCDT_Card,			//Video capture card
	JCDT_DVR,			//DVR
	JCDT_IPC,			//IPC
	JCDT_NVR,			//NVR
};

//Code type
enum JCCodecID
{
	JCCID_Unknown,		//Code unknown

	//Video code
	JCCID_JENC04,		//Jovision 04 edition coding
	JCCID_H264,			//H264 code

	//Audio code
	JCCID_PCM,			//PCM data
	JCCID_AMR,			//AMR code
	JCCID_G711A,		//G711 code A
	JCCID_G711U,		//G711 code U
};

//Stream Frame
typedef struct _JCStreamFrame
{
	JCFrameType sType;
	PUCHAR pFrameData;
	int nFrameSize;
}
JCStreamFrame, *PJCStreamFrame;

//Retrieve device information
typedef struct _JCLanDeviceInfo
{
	char szCloudSEE[16];
	char szIP[16];
	int nPort;
	char szDeviceName[128];
	int nDeviceType;
	int nChannelNum;
}
JCLanDeviceInfo, *PJCLanDeviceInfo;

//Device information
typedef struct _JCDeviceInfo
{
	JCDeviceType eDevType;
}
JCDeviceInfo, *PJCDeviceInfo;

//Stream information
typedef struct _JCStreamInfo
{
	JCCodecID eVideoCodec;		//Video code
	int nFrameWidth;			//Video frame width
	int nFrameHeight;			//Video frame highth
	int nVideoFrameRateNum;		//Video frame molecular
	int nVideoFrameRateDen;		//Video frame Denominator

	JCCodecID eAudioCodec;		//Audio code
	int nAudioSamples;			//Audio sample rate
	int nAudioSampleBits;		//Audio sample median
	int nAudioChannels;			//Audio channel

	JCRecFileType eRecFileType;	//Record file type
}
JCStreamInfo, *PJCStreamInfo;

//Date 
typedef struct _JCDateBlock
{
	int nBeginYear;		//Start year
	int nBeginMonth;	//Start month
	int nBeginDay;		//Start date
	int nEndYear;		//End year
	int nEndMonth;		//End month
	int nEndDay;		//End date
}
JCDateBlock, *PJCDateBlock;

//Record information
typedef struct _JCRecFileInfo
{
	int nRecFileID;
	JCRecFileType eType;
	char szBeginTime[8];
	char szFilename[16];
	char szPathName[64];
}
JCRecFileInfo, *PJCRecFileInfo;

//yv12 Data frame
typedef struct _JCRawFrame
{
	DWORD dwWidth;
	DWORD dwHeight;
	PUCHAR pY;
	PUCHAR pU;
	PUCHAR pV;
	DWORD dwYPitch;
	DWORD dwUPitch;
	DWORD dwVPitch;
}
JCRawFrame, *PJCRawFrame;

//Callback function
typedef void (__cdecl *JCEventCallback_t)(JCLink_t nLinkID, JCEventType etType, DWORD_PTR pData1, DWORD_PTR pData2, LPVOID pUserData);
typedef void (__cdecl *JCDataCallback_t)(JCLink_t nLinkID, PJCStreamFrame pFrame, LPVOID pUserData);
typedef void (__cdecl *JCRawDataCallback_t)(JCLink_t nLinkID, PJCRawFrame pFrame, LPVOID pUserData);
typedef void (__cdecl *JCLanSearchDeviceCallback_t)(PJCLanDeviceInfo pDevice);

//Whole fuction------------------------------------------------------------------------------
//Initialization SDK
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_InitSDK(int nLocStartPort);

//Release SDK
JCSDK_API void JCSDK_CALLTYPE JCSDK_ReleaseSDK();

//Set callback function
JCSDK_API void JCSDK_CALLTYPE JCSDK_RegisterCallback(
	JCEventCallback_t funEventCallback,
	JCDataCallback_t funDataCallback,
	JCRawDataCallback_t funRawDataCallback,
	JCLanSearchDeviceCallback_t funLanSearchDeviceCallback);

//connecting-----------------------------------------------------------------------------------
//connecting
JCSDK_API JCLink_t JCSDK_CALLTYPE JCSDK_Connect(char *szServer, int nPort, int nChannel,
	char *szNetUser, char *szPwd, BOOL bNumOrIP, LPVOID pUserData);

//End of connection
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_Disconnect(JCLink_t nLinkID);

//Get devoice information
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_GetDeviceInfo(JCLink_t nLinkID, PJCDeviceInfo pDevInfo);

//Decoding Preview-------------------------------------------------------------------------------
//Open the decoder
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_EnableDecoder(JCLink_t nLinkID, BOOL bEnable);

//Set video preview
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_SetVideoPreview(JCLink_t nLinkID, HWND hWnd, LPRECT pRect);

//Set audio preview
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_SetAudioPreview(JCLink_t nLinkID, HWND hWnd);

//Get stream information
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_GetStreamInfo(JCLink_t nLinkID, PJCStreamInfo pInfo);

//Record-------------------------------------------------------------------------------
//Start record
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_StartRec(JCLink_t nLinkID, char *szFilename);

//Close record
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_StopRec(JCLink_t nLinkID);

//Remote control-------------------------------------------------------------------------------
//Reserve remote video
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_GetRemoteRecFileList(JCLink_t nLinkID, PJCDateBlock pDate);

//Playback remote video
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_RemotePlay(JCLink_t nLinkID, int nFileID);

//Download remote video
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_DownloadRemoteFile(JCLink_t nLinkID, int nFileID);

JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_RemoteRecording(JCLink_t nLinkID, BOOL bEnable);
//other-----------------------------------------------------------------------------------
//Search LAN device
JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_LanSeartchDevice(int nDeviceType, DWORD dwTimeout);

JCSDK_API BOOL JCSDK_CALLTYPE JCSDK_SaveBitmap(JCLink_t nLinkID, const char *szFilename);

JCSDK_API void JCSDK_CALLTYPE JCSDK_yv12_to_rgb32(PUCHAR pDest, DWORD dwDestPitch, PUCHAR pSrcY, PUCHAR pSrcU, PUCHAR pSrcV, 
	DWORD dwSrcYPitch, DWORD dwSrcUPitch, DWORD dwSrcVPitch, DWORD dwWidth, DWORD dwHeight);
